<?php

declare(strict_types=1);

namespace Drupal\ip_info\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides a service for interacting with the ipapi.co API.
 */
class IpApiCo implements IpApiCoInterface {

  use StringTranslationTrait;


  /**
   * The base URL for the ipapi.co API.
   */
  protected const API_BASE_URL = 'https://ipapi.co/';

  /**
   * The IP Info base service.
   *
   * @var \Drupal\ip_info\Services\IpInfoInterface
   */
  protected IpInfoInterface $ipInfoBase;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new IpInfoAi service.
   *
   * @param \Drupal\ip_info\Services\IpInfoInterface $ip_info_base
   *   The IP Info base service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    IpInfoInterface $ip_info_base,
    ConfigFactoryInterface $config_factory,
  ) {
    $this->ipInfoBase = $ip_info_base;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function getProcessedData(string $ip_address): array {
    $data = [];
    $settings = Settings::get('ip_info');
    $config = $this->configFactory->get('ip_info.settings');
    if ($settings && array_key_exists('ipapi_co_enable', $settings) && $settings['ipapi_co_enable'] === TRUE) {
      $enable = TRUE;
    }
    else {
      $enable = $config->get('ipapi_co_enable');
    }

    if (!$enable) {
      return $data;
    }

    $url = self::API_BASE_URL . $ip_address . '/json';

    $ip_information = $this->ipInfoBase->getIpInformation($ip_address, $url, 'ipapi.co');

    if ($ip_information) {
      if (array_key_exists('city', $ip_information)) {
        $location_info = $ip_information['city'];
        if (array_key_exists('region', $ip_information)) {
          $location_info .= ', ' . $ip_information['region'];
        }
        if (array_key_exists('country_name', $ip_information)) {
          $location_info .= ', ' . $ip_information['country_name'];
        }
        if (array_key_exists('in_eu', $ip_information) && $ip_information['in_eu'] === TRUE) {
          $location_info .= ' (EU)';
        }
        $data[] = ['label' => $this->t('Location'), 'value' => $location_info];
      }
      if (array_key_exists('org', $ip_information)) {
        $organization_info = $ip_information['org'];
        if (array_key_exists('asn', $ip_information)) {
          $organization_info = $ip_information['asn'] . ' ' . $organization_info;
        }
        $data[] = ['label' => $this->t('Organization'), 'value' => $organization_info];
      }
    }

    return $data;
  }

}
