<?php

declare(strict_types=1);

namespace Drupal\ip_info\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides a service for interacting with the ipinfo.io API.
 */
class IpInfoIo implements IpInfoIoInterface {

  use StringTranslationTrait;


  /**
   * The base URL for the ipinfo.io API.
   */
  protected const API_BASE_URL = 'https://ipinfo.io/';

  /**
   * The IP Info base service.
   *
   * @var \Drupal\ip_info\Services\IpInfoInterface
   */
  protected IpInfoInterface $ipInfoBase;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new IpInfoAi service.
   *
   * @param \Drupal\ip_info\Services\IpInfoInterface $ip_info_base
   *   The IP Info base service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    IpInfoInterface $ip_info_base,
    ConfigFactoryInterface $config_factory,
  ) {
    $this->ipInfoBase = $ip_info_base;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function getProcessedData(string $ip_address): array {
    $data = [];
    $settings = Settings::get('ip_info');
    $config = $this->configFactory->get('ip_info.settings');
    if ($settings && array_key_exists('ipinfo_io_token', $settings)) {
      $token = $settings['ipinfo_io_token'];
    }
    else {
      $token = $config->get('ipinfo_io_token');
    }

    if (empty($token)) {
      return $data;
    }

    $url = self::API_BASE_URL . $ip_address . '?token=' . $token;

    $ip_information = $this->ipInfoBase->getIpInformation($ip_address, $url, 'ipinfo.io');

    if ($ip_information) {
      if (array_key_exists('country', $ip_information)) {
        $data[] = ['label' => $this->t('Country'), 'value' => $ip_information['country']];
      }
      if (array_key_exists('region', $ip_information)) {
        $data[] = ['label' => $this->t('Region'), 'value' => $ip_information['region']];
      }
      if (array_key_exists('city', $ip_information)) {
        $data[] = ['label' => $this->t('City'), 'value' => $ip_information['city']];
      }
      if (array_key_exists('org', $ip_information)) {
        $data[] = ['label' => $this->t('Organization'), 'value' => $ip_information['org']];
      }
      if (array_key_exists('hostname', $ip_information)) {
        $data[] = ['label' => $this->t('Hostname'), 'value' => $ip_information['hostname']];
      }
      if (array_key_exists('bogon', $ip_information)) {
        $data[] = ['label' => $this->t('This ip is'), 'value' => $this->t('Bogon')];
      }
    }

    return $data;
  }

}
